package dk.vajhoej.vms.indexf;

import dk.vajhoej.record.ArrayField;
import dk.vajhoej.record.FieldType;
import dk.vajhoej.record.Selector;
import dk.vajhoej.record.Struct;
import dk.vajhoej.record.StructField;
import dk.vajhoej.record.SubType;

/**
 * Class IndexFRecord represents an INDEXF.SYS record.
 */
@Struct
public class IndexFRecord {
	// **** start File Header
	@StructField(n=0,type=FieldType.UINT1)
	private int idOffset;
	@StructField(n=1,type=FieldType.UINT1)
	private int mpOffset;
	@StructField(n=2,type=FieldType.UINT1)
	private int acOffset;
	@StructField(n=3,type=FieldType.UINT1)
	private int rsOffset;
	@StructField(n=4,type=FieldType.INT2)
	private int segNum;
	@StructField(n=5,type=FieldType.INT1)
	private int strucLevMinor;
	@StructField(n=6,type=FieldType.INT1)
	@Selector(subtypes= { @SubType(value=2,type=IndexFRecord2.class),
                          @SubType(value=5,type=IndexFRecord5.class),
                          @SubType(value=0,type=IndexFRecordNone.class)})
	private int strucLevMajor;
	@ArrayField(elements=3)
	@StructField(n=7,type=FieldType.INT2)
	private short[] fid = new short[3];
	@ArrayField(elements=3)
	@StructField(n=8,type=FieldType.INT2)
	private short[] extFid = new short[3];
	@ArrayField(elements=32)
	@StructField(n=9,type=FieldType.INT1)
	private byte[] recAttr = new byte[32];
	@StructField(n=10,type=FieldType.INT4)
	private int fileChar;
	@ArrayField(elements=2)
	@StructField(n=11,type=FieldType.INT1)
	private byte[] reserved1 = new byte[2];
	@StructField(n=12,type=FieldType.INT1)
	private int mapInUse;
	@StructField(n=13,type=FieldType.INT1)
	private int accMode;
	@StructField(n=14,type=FieldType.INT4)
	private int fileOwner;
	@StructField(n=15,type=FieldType.INT2)
	private int fileProt;
	@ArrayField(elements=3)
	@StructField(n=16,type=FieldType.INT2)
	private short[] backLink = new short[3];
	@StructField(n=17,type=FieldType.INT1)
	private int journal;
	@StructField(n=18,type=FieldType.INT1)
	private int ruActive;
	@ArrayField(elements=2)
	@StructField(n=19,type=FieldType.INT1)
	private byte[] reserved2 = new byte[2];
	@StructField(n=20,type=FieldType.INT4)
	private int HighWater;
	// **** end File Header
	@ArrayField(elements=-1) // elements dynamically calculated
	@StructField(n=21,type=FieldType.INT1)
	private byte[] space1 = new byte[512];
	/**
	 * Get offset to ident area.
	 * @return offset
	 */
	public int getIdOffset() {
		return idOffset;
	}
	/**
	 * Get offset to map area.
	 * @return offset
	 */
	public int getMpOffset() {
		return mpOffset;
	}
	/**
	 * Get offset to ACL area.
	 * @return offset
	 */
	public int getAcOffset() {
		return acOffset;
	}
	/**
	 * Get offset to reserved area.
	 * @return offset
	 */
	public int getRsOffset() {
		return rsOffset;
	}
	/**
	 * Get segment number.
	 * @return segment number
	 */
	public int getSegNum() {
		return segNum;
	}
	/**
	 * Get structure level - minor,
	 * @return minor level
	 */
	public int getStrucLevMinor() {
		return strucLevMinor;
	}
	/**
	 * Get structure level - major (2 or 5)
	 * @return major level
	 */
	public int getStrucLevMajor() {
		return strucLevMajor;
	}
	/**
	 * Get FID.
	 * @return FID
	 */
	public short[] getFid() {
		return fid;
	}
	/**
	 * Get extended FID.
	 * @return extended FID
	 */
	public short[] getExtFid() {
		return extFid;
	}
	/**
	 * Get record attributes.
	 * @return record attributes
	 */
	public byte[] getRecAttr() {
		return recAttr;
	}
	/**
	 * Get file characteristics.
	 * @return file characteristics
	 */
	public int getFileChar() {
		return fileChar;
	}
	/**
	 * Get number map area words in use.
	 * @return words
	 */
	public int getMapInUse() {
		return mapInUse;
	}
	/**
	 * Get access priv level.
	 * @return level
	 */
	public int getAccMode() {
		return accMode;
	}
	/**
	 * Get file owner UIC.
	 * @return UIC
	 */
	public int getFileOwner() {
		return fileOwner;
	}
	/**
	 * Get file protection.
	 * @return protection mask
	 */
	public int getFileProt() {
		return fileProt;
	}
	/**
	 * Get backlink FID.
	 * @return FID
	 */
	public short[] getBackLink() {
		return backLink;
	}
	/**
	 * Get journal control flag.
	 * @return journal control flag
	 */
	public int getJournal() {
		return journal;
	}
	/**
	 * Get RU id.
	 * @return RU id
	 */
	public int getRuActive() {
		return ruActive;
	}
	/**
	 * Get highwater mark.
	 * @return highwater mark.
	 */
	public int getHighWater() {
		return HighWater;
	}
	/**
	 * Get record checksum.
	 * @return record checksum
	 */
	public short getChecksum() {
		throw new RuntimeException("getChecksum must be overwritten");
	}
	/**
	 * Get size of space after ident area before record checksum.
	 * @return size
	 */
	public int getSpace2Index() {
		return -1;
	}
	/**
	 * Get size of file header.
	 * @return size of file header
	 */
	public int getFileHeaderSize() {
		return 80;
	}
	/**
	 * Get size of ident area.
	 * @return size of ident area
	 */
	public int getIdentAreaSize() {
		throw new RuntimeException("getIdentAreaSize must be overwritten");
	}
	/**
	 * Is the record a valid and undeleted file.
	 * @return true if valid, false if non-valid
	 */
	public boolean isValidFile() {
		if(segNum != 0) return false;
		if((fileChar & 0x8000 ) != 0) return false;
		if((idOffset < 40)) return false;
		if((idOffset >= mpOffset)) return false;
		if(strucLevMajor != 2 && strucLevMajor != 5) return false;
		if(fid[0] == 0) return false;
		if(fid[1] == 0) return false;
		return true;
	}
}
